from PyQt6.QtCore import QThreadPool
from concurrent.futures import ThreadPoolExecutor
import threading
import time

# 创建一个全局的线程池实例
pyqt_thread_pool = QThreadPool.globalInstance()


class ThreadPoolManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    cls._instance = super(ThreadPoolManager, cls).__new__(cls)
                    max_workers = kwargs.get('max_workers', 5)
                    cls._instance.thread_pool = ThreadPoolExecutor(max_workers=max_workers)
        return cls._instance

    def get_thread_pool(self):
        return self.thread_pool


class RateLimiter:
    def __init__(self, rate: int, per: float):
        self._rate = rate
        self._per = per
        self._allowance = rate
        self._last_check = time.monotonic()
        self._lock = threading.Lock()

    def acquire(self):
        with self._lock:
            current = time.monotonic()
            time_passed = current - self._last_check
            self._last_check = current
            self._allowance += time_passed * (self._rate / self._per)
            if self._allowance > self._rate:
                self._allowance = self._rate
            if self._allowance < 1.0:
                sleep_time = (1.0 - self._allowance) * (self._per / self._rate)
                time.sleep(sleep_time)
                self._allowance = 0
                return
            self._allowance -= 1.0


# 创建全局实例
global_thread_pool = ThreadPoolManager(max_workers=20).get_thread_pool()
rate_limiter = RateLimiter(rate=5, per=1.0)
